type context = import('./config').context;

type sharedFields = import('./config').sharedFields;

type typeName = import('./config').typeName;

type typeObj<s> = { [key: string]: type<s> };

export type resolve = (arg: {
  context: context;
  input: any;
  object: { [key: string]: any };
  path: string[];
  query: { [key: string]: any };
  schema: schema;
  value: any;
}) => any;

export type type<s> =
  | keyof s
  | type<s>[]
  | ((
      | { optional: type<s> }
      | { nullable: type<s> }
      | { arrayOf: type<s>; minLength?: number; maxLength?: number }
      | { oneOf: typeObj<s>; resolveType: (value: any) => string }
      | { object: typeObj<s>; defaultType?: type }
      | {
          input?: type<s>;
          type?: type<s>;
          typeInput?: any;
          resolve?: resolve | object | string | number | boolean | null;
        }
    ) & {
      cost?:
        | number
        | ((arg: Parameters<resolve>[0] & { cost: number }) => number);
      defaultValue?: any;
      validate?: resolve;
    } & sharedFields);

export type schema = { [key in typeName]: type };

export type query<
  s extends { [k: string]: unknown },
  t extends type<s>
> = t extends keyof s
  ? query<s, s[t]>
  : t extends unknown[]
  ? { [k in keyof t]: query<s, t[k]> }
  : t extends { optional: unknown }
  ? query<s, t['optional']>
  : t extends { nullable: unknown }
  ? query<s, t['nullable']>
  : t extends { arrayOf: unknown }
  ? query<s, t['arrayOf']>
  : t extends { oneOf: unknown }
  ? Partial<{
      [k in keyof t['oneOf'] as k extends string ? `_on_${k}` : never]: query<
        s,
        t['oneOf'][k]
      >;
    }>
  : t extends { object: unknown }
  ? Partial<{ [k in keyof t['object']]: query<s, t['object'][k]> }>
  : t extends { type: unknown }
  ? query<s, t['type']>
  : Record<string, never>;

// export type value<t> = t extends typeName
//   ? value<s[t]>
//   : t extends any[]
//   ? { [k in keyof t]: value<t[k]> }
//   : t extends { optional: any }
//   ? value<t['optional']> | undefined
//   : t extends { nullable: any }
//   ? value<t['nullable']> | null | undefined
//   : t extends { arrayOf: any }
//   ? value<t['arrayOf']>
//   : t extends { oneOf: any }
//   ? Partial<{
//       [k in keyof t['oneOf'] as k extends string ? `_on_${k}` : never]: value<
//         t['oneOf'][k]
//       >;
//     }>
//   : t extends { object: any }
//   ? Partial<{ [k in keyof t['object']]: value<t['object'][k]> }>
//   : t extends { type: any }
//   ? value<t['type']>
//   : t;

export default {
  execute: (arg: {
    context?: any;
    path?: string[];
    query: object;
    schema: schema;
    type: type;
    value?: any;
  }) => any,
  getQueryCost: (arg: {
    context?: any;
    path?: string[];
    query: object;
    schema: schema;
    type: type;
  }) => number,
  validateQuery: (arg: {
    context?: any;
    path?: string[];
    query: object;
    schema: schema;
    type: type;
    value?: any;
  }) => number,
  validateSchema: (arg: {
    extensions: {
      optional?: typeObj;
      nullable?: typeObj;
      arrayOf?: typeObj;
      oneOf?: typeObj;
      object?: typeObj;
      resolve?: typeObj;
    };
    schema: schema;
  }) => number,
  validateValue: (arg: {
    context?: any;
    path?: string[];
    schema: schema;
    type: type;
    value: any;
  }) => any
};
